﻿Imports System.Reflection
Imports Syncfusion.UI.Xaml.Charts

'Custom date time category axis created for overriding the labels generation method.
Class CustomDateTimeCategoryXAxis
    Inherits DateTimeCategoryAxis

    Public MainWindow As MainWindow


    ' Overriding this method can generate the labels As required In accordance With the data.
    Protected Overrides Sub GenerateVisibleLabels()

        If IntervalType = DateTimeIntervalType.Days Then

            Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)
            area.Annotations.Clear()
            Dim actualSeries As ChartSeriesBase = MainWindow.SeriesPrice
            Dim actualXValues = CType(actualSeries.GetType().GetProperty("ActualXValues", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(actualSeries), System.Collections.IEnumerable)
            Dim XValues = CType(actualXValues, List(Of Double))

            If (Not (XValues) Is Nothing) AndAlso (XValues.Count > 0) Then

                Dim interval As Double = Me.Interval
                Dim position As Integer = VisibleRange.Start - VisibleRange.Start Mod interval
                Dim xStartValue = XValues(position).FromOADate()
                Dim nextDate As DateTime = xStartValue.Date.AddDays(CType(interval, Integer)).Date
                Dim nextHour As DateTime = New Date(xStartValue.Year, xStartValue.Month, xStartValue.Day, xStartValue.Hour + 1, 0, 0)

                Do While position <= VisibleRange.End

                    Dim xDateTime As DateTime = XValues(position).FromOADate()


                    If xDateTime.Date >= nextDate Then   'ADD NEW DATE LABEL
                        Dim labelString As String = xDateTime.ToString(Me.LabelFormat, Globalization.CultureInfo.CurrentCulture)
                        VisibleLabels.Add(New DateTimeAxisLabel(position, labelString, position))
                        nextDate = xDateTime.AddDays(CType(interval, Integer)).Date
                        nextHour = New Date(xDateTime.Year, xDateTime.Month, xDateTime.Day, xDateTime.Hour + 1, 0, 0)


                    Else   'ADD NEW HOUR LABEL
                        If xDateTime >= nextHour Then

                            If Not xDateTime.Hour = 16 Then 'do not add last hour of the day label to prevent intersection with the next date label
                                Dim labelString As String = xDateTime.ToString("HH", Globalization.CultureInfo.CurrentCulture)
                                Dim annotation As TextAnnotation = New TextAnnotation
                                annotation.X1 = position
                                'annotation.Y1 = MainWindow.chart.PointToValue(MainWindow.YAxisPrice, New Point(0, 14))
                                annotation.HorizontalAlignment = HorizontalAlignment.Center
                                annotation.VerticalAlignment = VerticalAlignment.Center
                                annotation.Foreground = New SolidColorBrush(Colors.White)
                                annotation.Text = labelString
                                area.Annotations.Add(annotation)

                            End If

                            nextHour = nextHour.AddHours(1)

                        End If

                    End If

                    position += 1
                Loop

            End If
        Else
            MyBase.GenerateVisibleLabels()
        End If


    End Sub

    'We need to update the secondary axis visible range after the primary axis actual range is updated.
    Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
            MyBase.CalculateVisibleRange(avalableSize)
            Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)

            For Each axis In area.Axes

                Dim customSecondaryAxis As Object = Nothing

                If TypeOf axis Is CustomLogarithmicYAxis Or TypeOf axis Is CustomNumericalYAxis Then
                    customSecondaryAxis = axis
                End If

                If Not customSecondaryAxis Is Nothing Then
                    If customSecondaryAxis.IsUpdateNeeded Then

                        customSecondaryAxis.ZoomY(Me.VisibleRange, area)
                        customSecondaryAxis.IsUpdateNeeded = False
                    End If

                End If
            Next

        End Sub

    End Class

Public Class CustomLogarithmicYAxis
    Inherits LogarithmicAxis

    Public prevXVisibleRange As DoubleRange
    Public prevVisibleRange As DoubleRange
    Public IsUpdateNeeded As Boolean = True

    Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
        MyBase.CalculateVisibleRange(avalableSize)

        Me.VisibleRange = prevVisibleRange

        Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)
        Dim primaryAxis = CType(area.PrimaryAxis, ChartAxisBase2D)
        Dim actualXRange = CType(primaryAxis.GetType().GetProperty("ActualRange", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(primaryAxis), DoubleRange)
        If actualXRange.IsEmpty Then Return
        Dim primaryAxisVisibleRange = CalculatePrimaryAxisRange(area, primaryAxis, actualXRange)

        If primaryAxisVisibleRange = prevXVisibleRange Then Return
        IsUpdateNeeded = True

        prevXVisibleRange = primaryAxisVisibleRange

    End Sub

    Private Function CalculatePrimaryAxisRange(area As SfChart, primaryAxis As ChartAxisBase2D, ByRef actualXRange As DoubleRange) As DoubleRange

        Dim rangeStart As Double
        rangeStart = actualXRange.Start + actualXRange.Delta * primaryAxis.ZoomPosition

        Dim rangeEnd As Double
        rangeEnd = rangeStart + primaryAxis.ZoomFactor * actualXRange.Delta

        Dim visibleXRange = New DoubleRange(rangeStart, rangeEnd)

        Return visibleXRange

    End Function

    Public Sub ZoomY(visibleXRange As DoubleRange, area As SfChart)
        Dim MinLogY, MaxLogY As Double
        Dim Ymax, Ymin As Double

        GetExtremes(visibleXRange.Start, visibleXRange.End, Ymax, Ymin, area, Me)

        MinLogY = Math.Log(Ymin, LogBase)
        MaxLogY = Math.Log(Ymax, LogBase)

        Me.VisibleRange = New DoubleRange(MinLogY, MaxLogY)

        prevVisibleRange = Me.VisibleRange

        '    Console.WriteLine("CustomLogarithmicYAxis " & Me.Name & "  ZoomY VisibleRange.Start " & Me.VisibleRange.Start & " VisibleRange.End " & Me.VisibleRange.End)
    End Sub

    Protected Overrides Sub GenerateVisibleLabels()
        MyBase.GenerateVisibleLabels()

        Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)

        For Each annotation In area.Annotations
            annotation.Y1 = area.PointToValue(Me, New Point(0, 14))
        Next

    End Sub

End Class

Class CustomNumericalYAxis
        Inherits NumericalAxis

        Public prevVisibleRange As DoubleRange
        Public IsUpdateNeeded As Boolean = True

        Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
            MyBase.CalculateVisibleRange(avalableSize)

            ' Console.WriteLine("CustomNumericalYAxis " & Me.Name & " CalculateVisibleRange VisibleRange.Start " & Me.VisibleRange.Start & " VisibleRange.End " & Me.VisibleRange.End)
            '  Console.WriteLine("CustomNumericalYAxis " & Me.Name & " CalculateVisibleRange prevVisibleRange.Start " & Me.prevVisibleRange.Start & " prevVisibleRange.End " & Me.prevVisibleRange.End)

            Me.VisibleRange = prevVisibleRange

        End Sub


        Public Sub ZoomY(visibleXRange As DoubleRange, area As SfChart)
            Dim Ymax, Ymin As Double

            GetExtremes(visibleXRange.Start, visibleXRange.End, Ymax, Ymin, area, Me)

            Me.VisibleRange = New DoubleRange(0, Ymax)

            prevVisibleRange = Me.VisibleRange

            ' Console.WriteLine("CustomNumericalYAxis " & Me.Name & "  ZoomY VisibleRange.Start " & Me.VisibleRange.Start & " VisibleRange.End " & Me.VisibleRange.End)
        End Sub

    End Class

Public Class FastSteplineBitmapSeriesExt
    Inherits FastStepLineBitmapSeries
    Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
        MyBase.SetIndividualPoint(index, obj, replace)
        CType(Me.YAxis, Object).IsUpdateNeeded = True
    End Sub
End Class

Class AreaSeriesExt
        Inherits AreaSeries
        Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
            MyBase.SetIndividualPoint(index, obj, replace)
            CType(Me.YAxis, Object).IsUpdateNeeded = True
        End Sub
    End Class

    Class FastRangeAreaBitmapSeriesExt
        Inherits FastRangeAreaBitmapSeries
        Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
            MyBase.SetIndividualPoint(index, obj, replace)
            CType(Me.YAxis, Object).IsUpdateNeeded = True
        End Sub
    End Class

